//rotatePolygon (C) 2016 Loe Feijs and TU/e for Golden Ratio Module.
//The program allows editing of a four-point polygon. Edit functions:
// 1) drag the four points
// 2) rotate the polygon by changing the angle between gravity-center and green dot.
//The inside area is filled with randomly positioned pink "blobs".

//The purpose of the program is to show a rotation matrix in action
//and combine this with a generate-and test approach.

import processing.pdf.*;

final int nrPoints = 4;       //how many points already created
float[][] myPoints = new float[nrPoints][2]; //polygon points 
float[]   myHandle  = new float[2];//green handle to rotate
float theta = 0;      //for rotation
float prev_theta = 0; //previous rotation

void setup(){
     size(500,500);
     background(255);
     stroke(0);
     //four points:
     myPoints[0][0] = 150;      //x coordinate first point
     myPoints[0][1] = 150;      //y coordinate idem
     myPoints[1][0] = 300;      //x coordinate second point
     myPoints[1][1] = 150;      //y coordinate idem
     myPoints[2][0] = 300;      //x coordinate third
     myPoints[2][1] = 300;      //y
     myPoints[3][0] = 150;      //x coordinate 4th
     myPoints[3][1] = 300;      //y 
     myHandle[0] = 375;
     myHandle[1] = 225;
}

void makePolygon(float[][] p){
     //draw the polygon as connected line segments
     //p must contain four points, each as x,y mini-array
     stroke(0);
     noFill();
     //draw the lines first, going round
     for (int i=0; i < nrPoints; i++){
          line(p[i][0],p[i][1],p[(i+1)%4][0],p[(i+1)%4][1]);
     }
     //draw the points  as little circles
     for (int i=0; i<nrPoints; i++)
          ellipse(myPoints[i][0],myPoints[i][1],4,4);
}
void myHook(float[] h){
     //draw it green
     stroke(0,100,0);
     fill(0,225,0);
     ellipse(h[0],h[1],4,4);
}
void makeBlob(int x, int y){
     //fink circle in generate-and-test approach
     stroke(255,0,0);
     fill(255,200,200);
     //test whether (x,y) is inside the polygon use testline xy1-xy2
     //count number of intersections
     int count = 0;
     float x1 = x;
     float y1 = y;
     float x2 = 5000; //very far away
     float y2 = y + 1000;  //far away
     for (int i=0; i< nrPoints; i++)
          if (intersects(x1,y1,x2,y2,myPoints[i][0],myPoints[i][1],myPoints[(i+1)%4][0],myPoints[(i+1)%4][1]))
              count++;
     if (count%2 == 1)
         //odd number of intersects means "inside"
         ellipse(x,y,5,5); 
}
void draw(){
     fill(100);
     frameRate(200);
     makePolygon(myPoints);
     myHook(myHandle);
     if (mousePressed){
         for (int i=0; i<nrPoints; i++){
              //drag the point being near the mouse
              //oops: keep them at least 20 pixels apart
              if (dist(myPoints[i][0],myPoints[i][1],mouseX,mouseY) < 20){
                       myPoints[i][0] = mouseX;
                       myPoints[i][1] = mouseY;
              }
         }
         //also move the green "handle" dot
         if (dist(myHandle[0],myHandle[1],mouseX,mouseY) < 20){
                  myHandle[0] = mouseX;
                  myHandle[1] = mouseY;
         }
     }
     makeBlob(int(random(width)),int(random(height)));
}

void mouseReleased(){
     //now it's time to execute the rotation,
     //first find cC,cY, being the center of gravity of the polygon
     float cX = (myPoints[0][0] + myPoints[1][0] + myPoints[2][0] + myPoints[3][0])/4;
     float cY = (myPoints[0][1] + myPoints[1][1] + myPoints[2][1] + myPoints[3][1])/4;
     //determine the direction of the line from gravity towards green handle
     theta = atan2(myHandle[1] - cY, myHandle[0] - cX);
     float dtheta = theta - prev_theta; 
     prev_theta = theta;
     println(degrees(dtheta));  
     for (int i = 0; i < 4; i++){
          //rotate each point around (cX,cY)
          float X = myPoints[i][0];
          float Y = myPoints[i][1];
          //first subtract (cX,xY), then rotate, then add (cX,cY) back again
          myPoints[i][0] = cX + (X - cX) * cos(dtheta) - (Y - cY) * sin(dtheta);
          myPoints[i][1] = cY + (X - cX) * sin(dtheta) + (Y - cY) * cos(dtheta);    
     }    //end for 
     background(255); 
}

void keyPressed() {
     beginRecord(PDF,"polygon" + str(int(random(1000000))) + ".pdf");
     //now generate the blobs again and write them to file
     makePolygon(myPoints);
     int blobAttempts = 10000;
     for (int k=0; k< blobAttempts; k++)
          makeBlob(int(random(width)),int(random(height)));
     endRecord();
     exit();
}
